function [psi,Effq] = velopt(psi0,src,kappa,q,L,scalefac)

% Problem parameters for Matlab's optimizer fmincon.
psi0 = psi0(:); problem.x0 = psi0(2:end);
problem.objective = @(x) normHq2(x,src,kappa,q,L,scalefac);
problem.nonlcon = @(x) nonlcon(x,src,kappa,q,L,scalefac);
problem.solver = 'fmincon';
problem.options = optimset('Display','iter','TolFun',1e-10,...
			   'GradObj','on','GradConstr','on',...
			   'algorithm','interior-point');

[psi,Hq2] = fmincon(problem);

% Mixing efficiency: call normHq2 with no flow to get pure-conduction solution.
Effq = sqrt(normHq2(zeros(size(psi)),src,kappa,q,L,scalefac) / Hq2);

psi = reshape([0;psi],size(src)); % Convert psi back into a square grid

%=======================================================================
function [varargout] = normHq2(psi,src,kappa,q,L,scalefac)

N = size(src,1); src = src(:);

% 2D Differentiation matrices and negative-Laplacian
[Dx,Dy,Dxx,Dyy] = Diffmat2(N,L); mlap = -(Dxx+Dyy);
if q ~= 0 && q ~= -1, error('This code only supports q = 0 or -1.'); end

psi = [0;psi]; ux = Dy*psi; uy = -Dx*psi;
ugradop = diag(sparse(ux))*Dx + diag(sparse(uy))*Dy;

if q == 0
  Aop2 = (-ugradop + kappa*mlap);
elseif q == -1
  Aop2 = mlap*(-ugradop + kappa*mlap);
end
Aop1 = (ugradop + kappa*mlap)*Aop2;
% Solve for chi, dropping corner point to fix normalisation.
chi = [0; Aop1(2:end,2:end) \ src(2:end)];
theta = Aop2*chi;

% The squared H^q norm of theta.
varargout{1} = L^2*sum(theta.^2)/N^2 * scalefac;

if nargout > 1
  % Gradient of squared-norm Hq2.
  gradHq2 = 2*((Dx*theta).*(Dy*chi) - (Dy*theta).*(Dx*chi));
  varargout{2} = gradHq2(2:end) / N^2 * scalefac;
end

%=======================================================================
function [c,ceq,gc,gceq] = nonlcon(psi,src,kappa,q,L,scalefac)

psi = [0;psi]; N = size(src,1);
c = []; gc = [];

[Dx,Dy,Dxx,Dyy] = Diffmat2(N,L); % 2D Differentiation matrices
U2 = L^2*(sum((Dx*psi).^2 + (Dy*psi).^2)/N^2);
ceq(1) = (U2-1) * scalefac;

if nargout > 2
  % Gradient of constraints
  mlappsi = -(Dxx+Dyy)*psi;
  gceq(:,1) = 2*mlappsi(2:end) / N^2 * scalefac;
end
